@@ -73,6 +73,18 @@ void empathy_account_widget_discard_pending_changes
 gchar * empathy_account_widget_get_default_display_name (
     EmpathyAccountWidget *widget);
 
+void empathy_account_widget_set_account_param (EmpathyAccountWidget *widget,
+    const gchar *account);
+
+void empathy_account_widget_set_password_param (EmpathyAccountWidget *self,
+    const gchar *password);
+
+void empathy_account_widget_set_other_accounts_exist (
+    EmpathyAccountWidget *self, gboolean others_exist);
+
+/* protected methods */
+void empathy_account_widget_changed (EmpathyAccountWidget *widget);
+
 G_END_DECLS
 
 #endif /* __EMPATHY_ACCOUNT_WIDGET_H__ */
