@@ -29,16 +29,11 @@
 
 G_BEGIN_DECLS
 
-void empathy_geometry_save (GtkWindow *window,
-    const gchar *name);
-
-void empathy_geometry_load (GtkWindow *window,
-    const gchar *name);
-
 void empathy_geometry_bind (GtkWindow *window,
     const gchar *name);
 
-void empathy_geometry_unbind (GtkWindow *window);
+void empathy_geometry_unbind (GtkWindow *window,
+    const gchar *name);
 
 G_END_DECLS
 
