@@ -510,8 +510,8 @@ empathy_contact_set_presence_message (EmpathyContact *contact,
 
   if (priv->persona != NULL)
     {
-      folks_presence_owner_set_presence_message (
-          FOLKS_PRESENCE_OWNER (priv->persona), message);
+      folks_presence_details_set_presence_message (
+          FOLKS_PRESENCE_DETAILS (priv->persona), message);
     }
 }
 
@@ -729,12 +729,12 @@ empathy_contact_set_alias (EmpathyContact *contact,
 
   /* Set the alias on the persona if possible */
   persona = empathy_contact_get_persona (contact);
-  if (persona != NULL && FOLKS_IS_ALIASABLE (persona))
+  if (persona != NULL && FOLKS_IS_ALIAS_DETAILS (persona))
     {
       DEBUG ("Setting alias for contact %s to %s",
           empathy_contact_get_id (contact), alias);
 
-      folks_aliasable_set_alias (FOLKS_ALIASABLE (persona), alias);
+      folks_alias_details_set_alias (FOLKS_ALIAS_DETAILS (persona), alias);
     }
 
   if (tp_strdiff (alias, priv->alias))
@@ -752,10 +752,10 @@ groups_change_group_cb (GObject *source,
     GAsyncResult *result,
     gpointer user_data)
 {
-  FolksGroupable *groupable = FOLKS_GROUPABLE (source);
+  FolksGroupDetails *group_details = FOLKS_GROUP_DETAILS (source);
   GError *error = NULL;
 
-  folks_groupable_change_group_finish (groupable, result, &error);
+  folks_group_details_change_group_finish (group_details, result, &error);
   if (error != NULL)
     {
       g_warning ("failed to change group: %s", error->message);
@@ -779,9 +779,9 @@ empathy_contact_change_group (EmpathyContact *contact, const gchar *group,
   persona = empathy_contact_get_persona (contact);
   if (persona != NULL)
     {
-      if (FOLKS_IS_GROUPABLE (persona))
-        folks_groupable_change_group (FOLKS_GROUPABLE (persona), group, is_member,
-          groups_change_group_cb, contact);
+      if (FOLKS_IS_GROUP_DETAILS (persona))
+        folks_group_details_change_group (FOLKS_GROUP_DETAILS (persona), group,
+            is_member, groups_change_group_cb, contact);
       return;
     }
 
@@ -944,7 +944,8 @@ empathy_contact_set_persona (EmpathyContact *contact,
   /* Set the persona's groups */
   if (priv->groups != NULL)
     {
-      folks_groupable_set_groups (FOLKS_GROUPABLE (persona), priv->groups);
+      folks_group_details_set_groups (FOLKS_GROUP_DETAILS (persona),
+          priv->groups);
       g_hash_table_destroy (priv->groups);
       priv->groups = NULL;
     }
@@ -991,8 +992,8 @@ empathy_contact_get_presence_message (EmpathyContact *contact)
   priv = GET_PRIV (contact);
 
   if (priv->persona != NULL)
-    return folks_presence_owner_get_presence_message (
-        FOLKS_PRESENCE_OWNER (priv->persona));
+    return folks_presence_details_get_presence_message (
+        FOLKS_PRESENCE_DETAILS (priv->persona));
 
   if (priv->tp_contact != NULL)
     return tp_contact_get_presence_message (priv->tp_contact);
@@ -1161,12 +1162,17 @@ static gboolean
 contact_has_log (EmpathyContact *contact)
 {
   TplLogManager *manager;
+  TplEntity *entity;
   gboolean have_log;
 
   manager = tpl_log_manager_dup_singleton ();
+  entity = tpl_entity_new (empathy_contact_get_id (contact),
+      TPL_ENTITY_CONTACT, NULL, NULL);
+
   have_log = tpl_log_manager_exists (manager,
-      empathy_contact_get_account (contact), empathy_contact_get_id (contact),
-      FALSE);
+      empathy_contact_get_account (contact), entity, TPL_EVENT_MASK_TEXT);
+
+  g_object_unref (entity);
   g_object_unref (manager);
 
   return have_log;
@@ -1715,6 +1721,13 @@ tp_caps_to_capabilities (TpCapabilities *caps)
                     TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_VIDEO))
                 capabilities |= EMPATHY_CAPABILITIES_VIDEO;
             }
+
+          if (tp_asv_get_boolean (fixed_prop,
+                    TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_AUDIO, NULL))
+            capabilities |= EMPATHY_CAPABILITIES_AUDIO;
+          if (tp_asv_get_boolean (fixed_prop,
+                    TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_VIDEO, NULL))
+            capabilities |= EMPATHY_CAPABILITIES_VIDEO;
         }
     }
 
@@ -1795,16 +1808,16 @@ static int
 presence_cmp_func (EmpathyContact *a,
     EmpathyContact *b)
 {
-  FolksPresenceOwner *presence_a, *presence_b;
+  FolksPresenceDetails *presence_a, *presence_b;
 
-  presence_a = FOLKS_PRESENCE_OWNER (empathy_contact_get_persona (a));
-  presence_b = FOLKS_PRESENCE_OWNER (empathy_contact_get_persona (b));
+  presence_a = FOLKS_PRESENCE_DETAILS (empathy_contact_get_persona (a));
+  presence_b = FOLKS_PRESENCE_DETAILS (empathy_contact_get_persona (b));
 
   /* We negate the result because we're sorting in reverse order (i.e. such that
    * the Personas with the highest presence are at the beginning of the list. */
-  return -folks_presence_owner_typecmp (
-      folks_presence_owner_get_presence_type (presence_a),
-      folks_presence_owner_get_presence_type (presence_b));
+  return -folks_presence_details_typecmp (
+      folks_presence_details_get_presence_type (presence_a),
+      folks_presence_details_get_presence_type (presence_b));
 }
 
 static gint
