@@ -1500,6 +1500,28 @@ accounts_dialog_get_account_iter (EmpathyAccountsDialog *dialog,
   return FALSE;
 }
 
+static void
+select_and_scroll_to_iter (EmpathyAccountsDialog *dialog,
+    GtkTreeIter *iter)
+{
+  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  GtkTreeSelection *selection;
+  GtkTreePath *path;
+  GtkTreeModel *model;
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
+
+  gtk_tree_selection_select_iter (selection, iter);
+
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
+  path = gtk_tree_model_get_path (model, iter);
+
+  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (priv->treeview), path, NULL,
+      TRUE, 0, 0.5);
+
+  gtk_tree_path_free (path);
+}
+
 static void
 accounts_dialog_model_set_selected (EmpathyAccountsDialog *dialog,
     EmpathyAccountSettings *settings)
@@ -1510,7 +1532,7 @@ accounts_dialog_model_set_selected (EmpathyAccountsDialog *dialog,
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
   if (accounts_dialog_get_settings_iter (dialog, settings, &iter))
-    gtk_tree_selection_select_iter (selection, &iter);
+    select_and_scroll_to_iter (dialog, &iter);
 }
 
 static void
@@ -1752,6 +1774,7 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
   const gchar        *name;
   gboolean            enabled;
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  gboolean selected = FALSE;
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
   status = tp_account_get_connection_status (account, NULL);
@@ -1761,7 +1784,16 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
   settings = empathy_account_settings_new_for_account (account);
 
   if (!accounts_dialog_get_account_iter (dialog, account, &iter))
-    gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+    {
+      gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+    }
+  else
+    {
+      GtkTreeSelection *selection;
+
+      selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
+      selected = gtk_tree_selection_iter_is_selected (selection, &iter);
+    }
 
   gtk_list_store_set (GTK_LIST_STORE (model), &iter,
       COL_NAME, name,
@@ -1770,6 +1802,14 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
       COL_ACCOUNT_SETTINGS, settings,
       -1);
 
+  if (selected)
+    {
+      /* We just modified the selected account. Its display name may have been
+       * changed and so it's place in the treeview. Scroll to it so it stays
+       * visible. */
+      select_and_scroll_to_iter (dialog, &iter);
+    }
+
   accounts_dialog_connection_changed_cb (account,
       0,
       status,
@@ -1943,20 +1983,7 @@ accounts_dialog_set_selected_account (EmpathyAccountsDialog *dialog,
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
   if (accounts_dialog_get_account_iter (dialog, account, &iter))
-    {
-      GtkTreePath *path;
-      GtkTreeModel *model;
-
-      gtk_tree_selection_select_iter (selection, &iter);
-
-      model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
-      path = gtk_tree_model_get_path (model, &iter);
-
-      gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (priv->treeview), path, NULL,
-          TRUE, 0, 0.5);
-
-      gtk_tree_path_free (path);
-    }
+    select_and_scroll_to_iter (dialog, &iter);
 }
 
 static void
@@ -1994,7 +2021,9 @@ accounts_dialog_cms_prepare_cb (GObject *source,
   if (!empathy_connection_managers_prepare_finish (cms, result, NULL))
     goto out;
 
-  accounts_dialog_update_settings (dialog, NULL);
+  /* No need to update the settings if we are already preparing one */
+  if (priv->settings_ready == NULL)
+    accounts_dialog_update_settings (dialog, NULL);
 
   if (priv->initial_selection != NULL)
     {
