@@ -20,7 +20,7 @@
 #include <telepathy-glib/telepathy-glib.h>
 
 #include <libempathy/empathy-chatroom-manager.h>
-#include <libempathy/empathy-dispatcher.h>
+#include <libempathy/empathy-request-util.h>
 #include <libempathy/empathy-utils.h>
 
 #include "empathy-chat-window.h"
@@ -286,12 +286,14 @@ empathy_chat_manager_init (EmpathyChatManager *self)
   priv->chatroom_mgr = empathy_chatroom_manager_dup_singleton (NULL);
 
   /* Text channels handler */
-  priv->handler = tp_simple_handler_new (dbus, FALSE, FALSE, "Empathy.Chat",
-      FALSE, handle_channels, self, NULL);
+  priv->handler = tp_simple_handler_new (dbus, FALSE, FALSE,
+      EMPATHY_CHAT_BUS_NAME_SUFFIX, FALSE, handle_channels, self, NULL);
 
-  /* EmpathyTpChat relies on this feature being prepared */
+  /* EmpathyTpChat relies on these features being prepared */
   tp_base_client_add_connection_features_varargs (priv->handler,
     TP_CONNECTION_FEATURE_CAPABILITIES, 0);
+  tp_base_client_add_channel_features_varargs (priv->handler,
+      TP_CHANNEL_FEATURE_CHAT_STATES, 0);
 
   g_object_unref (dbus);
 
@@ -431,10 +433,10 @@ empathy_chat_manager_undo_closed_chat (EmpathyChatManager *self)
       data->room ? "room" : "contact", data->id);
 
   if (data->room)
-    empathy_dispatcher_join_muc (data->account, data->id,
+    empathy_join_muc (data->account, data->id,
         TP_USER_ACTION_TIME_NOT_USER_ACTION);
   else
-    empathy_dispatcher_chat_with_contact_id (data->account, data->id,
+    empathy_chat_with_contact_id (data->account, data->id,
         TP_USER_ACTION_TIME_NOT_USER_ACTION);
 
   g_signal_emit (self, signals[CLOSED_CHATS_CHANGED], 0,
