@@ -36,7 +36,7 @@
 #include <libempathy/empathy-tp-roomlist.h>
 #include <libempathy/empathy-chatroom.h>
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-dispatcher.h>
+#include <libempathy/empathy-request-util.h>
 
 #include <libempathy-gtk/empathy-account-chooser.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
@@ -73,11 +73,6 @@ typedef struct {
 	GtkWidget         *viewport_error;
 } EmpathyNewChatroomDialog;
 
-typedef struct {
-  EmpathyAccountChooserFilterResultCallback callback;
-  gpointer                                  user_data;
-} FilterCallbackData;
-
 enum {
 	COL_NEED_PASSWORD,
 	COL_INVITE_ONLY,
@@ -138,70 +133,6 @@ static void	new_chatroom_dialog_button_close_error_clicked_cb   (GtkButton
 static EmpathyNewChatroomDialog *dialog_p = NULL;
 
 
-static void
-conn_prepared_cb (GObject *conn,
-		  GAsyncResult *result,
-		  gpointer user_data)
-{
-	FilterCallbackData *data = user_data;
-	GError             *myerr = NULL;
-	TpCapabilities     *caps;
-
-	if (!tp_proxy_prepare_finish (conn, result, &myerr)) {
-		data->callback (FALSE, data->user_data);
-		g_slice_free (FilterCallbackData, data);
-	}
-
-	caps = tp_connection_get_capabilities (TP_CONNECTION (conn));
-	data->callback (tp_capabilities_supports_text_chatrooms (caps),
-			data->user_data);
-
-	g_slice_free (FilterCallbackData, data);
-}
-
-/**
- * empathy_account_chooser_filter_supports_multichat:
- * @account: a #TpAccount
- * @callback: an #EmpathyAccountChooserFilterResultCallback accepting the result
- * @callback_data: data passed to the @callback
- * @user_data: user data or %NULL
- *
- * An #EmpathyAccountChooserFilterFunc that returns accounts that both
- * support multiuser text chat and are connected.
- *
- * Returns (via the callback) TRUE if @account both supports muc and is connected
- */
-static void
-empathy_account_chooser_filter_supports_multichat (
-	TpAccount                                 *account,
-	EmpathyAccountChooserFilterResultCallback  callback,
-	gpointer                                   callback_data,
-	gpointer                                   user_data)
-{
-	TpConnection       *connection;
-	FilterCallbackData *cb_data;
-	GQuark              features[] = { TP_CONNECTION_FEATURE_CAPABILITIES, 0 };
-
-	if (tp_account_get_connection_status (account, NULL) !=
-	    TP_CONNECTION_STATUS_CONNECTED) {
-		callback (FALSE, callback_data);
-		return;
-	}
-
-	/* check if CM supports multiuser text chat */
-	connection = tp_account_get_connection (account);
-	if (connection == NULL) {
-		callback (FALSE, callback_data);
-		return;
-	}
-
-	cb_data = g_slice_new0 (FilterCallbackData);
-	cb_data->callback = callback;
-	cb_data->user_data = callback_data;
-        tp_proxy_prepare_async (connection, features, conn_prepared_cb,
-		cb_data);
-}
-
 void
 empathy_new_chatroom_dialog_show (GtkWindow *parent)
 {
@@ -270,7 +201,7 @@ empathy_new_chatroom_dialog_show (GtkWindow *parent)
 	/* Account chooser for custom */
 	dialog->account_chooser = empathy_account_chooser_new ();
 	empathy_account_chooser_set_filter (EMPATHY_ACCOUNT_CHOOSER (dialog->account_chooser),
-					    empathy_account_chooser_filter_supports_multichat,
+					    empathy_account_chooser_filter_supports_chatrooms,
 					    NULL);
 	gtk_table_attach_defaults (GTK_TABLE (dialog->table_info),
 				   dialog->account_chooser,
@@ -507,8 +438,7 @@ new_chatroom_dialog_account_changed_cb (GtkComboBox             *combobox,
 	gboolean               listing = FALSE;
 	gboolean               expanded = FALSE;
 	TpConnection          *connection;
-	EmpathyDispatcher     *dispatcher;
-	GList                 *classes = NULL;
+	TpCapabilities *caps;
 
 	if (dialog->room_list) {
 		g_object_unref (dialog->room_list);
@@ -533,25 +463,21 @@ new_chatroom_dialog_account_changed_cb (GtkComboBox             *combobox,
 	dialog->status_changed_id = g_signal_connect (dialog->account,
 		      "status-changed", G_CALLBACK (account_status_changed_cb), dialog);
 
-	dispatcher = empathy_dispatcher_dup_singleton ();
+	/* empathy_account_chooser_filter_supports_chatrooms ensures that the
+	* account has a connection and CAPABILITIES has been prepared. */
+	g_assert (connection != NULL);
+	g_assert (tp_proxy_is_prepared (connection,
+		TP_CONNECTION_FEATURE_CAPABILITIES));
+	caps = tp_connection_get_capabilities (connection);
 
-	if (connection != NULL) {
-		classes = empathy_dispatcher_find_requestable_channel_classes (dispatcher,
-			connection, TP_IFACE_CHANNEL_TYPE_ROOM_LIST,
-			TP_HANDLE_TYPE_NONE, NULL);
-	}
-
-	if (classes != NULL) {
+	if (tp_capabilities_supports_room_list (caps, NULL)) {
 		/* Roomlist channels are supported */
 		dialog->room_list = empathy_tp_roomlist_new (dialog->account);
-		g_list_free (classes);
 	}
 	else {
 		dialog->room_list = NULL;
 	}
 
-	g_object_unref (dispatcher);
-
 	if (dialog->room_list) {
 		g_signal_connect (dialog->room_list, "destroy",
 				  G_CALLBACK (new_chatroom_dialog_roomlist_destroy_cb),
@@ -764,7 +690,7 @@ new_chatroom_dialog_join (EmpathyNewChatroomDialog *dialog)
 	g_strstrip (room_name);
 
 	DEBUG ("Requesting channel for '%s'", room_name);
-	empathy_dispatcher_join_muc (account, room_name,
+	empathy_join_muc (account, room_name,
 		gtk_get_current_event_time ());
 
 	g_free (room_name);
